"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = __importDefault(require("fs-extra"));
const inquirer_1 = __importDefault(require("inquirer"));
const path_1 = __importDefault(require("path"));
const ern_core_1 = require("ern-core");
const ApiImplGen_1 = __importDefault(require("./generators/ApiImplGen"));
async function generateApiImpl({ apiDependency, apiImplName, outputDirectory, nativeOnly, forceGenerate = false, reactNativeVersion, hasConfig = false, packageName, scope, paths, }) {
    ern_core_1.log.debug('Entering generate API IMPL');
    try {
        // get the directory to output the generated project.
        paths.outDirectory = outputDirectory = formOutputDirectoryName(packageName, outputDirectory);
        await createOutputDirectory(outputDirectory, forceGenerate);
        await createNodePackage(outputDirectory, apiDependency, apiImplName, packageName, nativeOnly, hasConfig, reactNativeVersion, scope);
        const platforms = getPlatforms(nativeOnly);
        await new ApiImplGen_1.default().generateApiImplementation(apiDependency, paths, reactNativeVersion, platforms);
    }
    catch (e) {
        ern_core_1.log.debug('command failed performing cleanup.');
        throw new Error(e);
    }
}
exports.default = generateApiImpl;
async function createOutputDirectory(outputDirectoryPath, forceGenerate) {
    if (!forceGenerate && (await fs_extra_1.default.pathExists(outputDirectoryPath))) {
        const { shouldRegenerate } = await inquirer_1.default.prompt([
            {
                default: false,
                message: `An implementation directory already exists in ${outputDirectoryPath}. Do you want to delete this and regenerate this project?`,
                name: 'shouldRegenerate',
                type: 'confirm',
            },
        ]);
        if (!shouldRegenerate) {
            throw Error('An implementation directory already exists');
        }
        else {
            forceGenerate = true;
        }
    }
    if (forceGenerate && (await fs_extra_1.default.pathExists(outputDirectoryPath))) {
        ern_core_1.log.info(`Deleting the existing directory and recreating a new one in ${outputDirectoryPath}`);
        ern_core_1.fileUtils.chmodr('755', outputDirectoryPath);
        ern_core_1.shell.rm('-Rf', outputDirectoryPath);
    }
    else {
        ern_core_1.log.debug(`creating output dir: ${outputDirectoryPath}`);
    }
    ern_core_1.shell.mkdir('-p', outputDirectoryPath);
}
async function createNodePackage(outputDirectoryPath, apiDependency, apiImplName, packageName, nativeOnly, hasConfig, reactNativeVersion, scope) {
    ern_core_1.shell.pushd(outputDirectoryPath);
    try {
        await ern_core_1.yarn.init();
        await ern_core_1.yarn.add(ern_core_1.PackagePath.fromString(`react-native@${reactNativeVersion}`), { dev: true });
        await ern_core_1.yarn.add(apiDependency);
        ern_core_1.shell.cp(path_1.default.join(ern_core_1.Platform.currentPlatformVersionPath, 'ern-api-impl-gen/resources/gitignore'), path_1.default.join(outputDirectoryPath, '.gitignore'));
        ernifyPackageJson(outputDirectoryPath, apiImplName, packageName, nativeOnly, hasConfig, reactNativeVersion, scope);
    }
    finally {
        ern_core_1.shell.popd();
    }
}
function formOutputDirectoryName(outputName, outputPath) {
    return path_1.default.join(outputPath ? outputPath : process.cwd(), outputName);
}
function getPlatforms(nativeOnly) {
    return nativeOnly ? ['android', 'ios'] : ['js'];
}
function ernifyPackageJson(outputDirectoryPath, apiImplName, packageName, nativeOnly, hasConfig, reactNativeVersion, scope) {
    const packageJson = (0, ern_core_1.readPackageJsonSync)(outputDirectoryPath);
    const moduleType = nativeOnly
        ? ern_core_1.ModuleTypes.NATIVE_API_IMPL
        : ern_core_1.ModuleTypes.JS_API_IMPL;
    const containerGen = {
        hasConfig,
        moduleName: ern_core_1.utils.camelize(apiImplName),
    };
    packageJson.name = scope ? `@${scope}/${packageName}` : packageName;
    packageJson.ern = {
        containerGen,
        moduleType,
    };
    if (nativeOnly) {
        packageJson.ern.pluginConfig = {
            android: {
                root: 'android/lib',
            },
            ios: {
                copy: [
                    {
                        dest: '{{{projectName}}}/APIImpls',
                        source: 'ios/ElectrodeApiImpl/APIImpls/*',
                    },
                ],
                pbxproj: {
                    addSource: [
                        {
                            from: 'ios/ElectrodeApiImpl/APIImpls/*.swift',
                            group: 'APIImpls',
                            path: 'APIImpls',
                        },
                    ],
                },
                requiresManualLinking: true,
            },
        };
    }
    packageJson.keywords
        ? packageJson.keywords.push(moduleType)
        : (packageJson.keywords = [moduleType]);
    (0, ern_core_1.writePackageJsonSync)(outputDirectoryPath, packageJson);
}
//# sourceMappingURL=ApiImpl.js.map