"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ROOT_DIR = void 0;
const ern_core_1 = require("ern-core");
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const fs_readdir_recursive_1 = __importDefault(require("fs-readdir-recursive"));
exports.ROOT_DIR = ern_core_1.shell.pwd();
const SRC_MAIN_JAVA_DIR = path_1.default.normalize('src/main/java');
const API_IMPL_PACKAGE = path_1.default.normalize('com/ern/api/impl');
class ApiImplAndroidGenerator {
    static getMustacheFileNamesMap(resourceDir, apiName) {
        const files = (0, fs_readdir_recursive_1.default)(resourceDir, (f) => f.endsWith('.mustache'));
        const classNames = {
            'RequestHandlerConfig.java.mustache': 'RequestHandlerConfig.java',
            'RequestHandlerProvider.java.mustache': 'RequestHandlerProvider.java',
            'apiController.mustache': `${apiName}ApiController.java`,
            'requestHandlerProvider.mustache': `${apiName}ApiRequestHandlerProvider.java`,
            'requestHandlers.mustache': `${apiName}ApiRequestHandler.java`,
        };
        return { files, classNames };
    }
    get name() {
        return 'ApiImplAndroidGenerator';
    }
    get platform() {
        return 'android';
    }
    async generate(apiDependency, paths, reactNativeVersion, plugins, apis, regen) {
        ern_core_1.log.debug(`Starting project generation for ${this.platform}`);
        this.regenerateApiImpl = regen;
        await this.fillHull(apiDependency, paths, reactNativeVersion, plugins, apis);
    }
    async fillHull(apiDependency, paths, reactNativeVersion, pluginsPaths, apis) {
        ern_core_1.shell.pushd(exports.ROOT_DIR);
        try {
            ern_core_1.log.debug(`[=== Starting hull filling for api impl gen for ${this.platform} ===]`);
            const outputDirectory = path_1.default.join(paths.outDirectory, 'android');
            ern_core_1.log.debug(`Creating out directory(${outputDirectory}) for android and copying container hull to it.`);
            fs_extra_1.default.ensureDirSync(outputDirectory);
            ern_core_1.fileUtils.chmodr('755', outputDirectory);
            ern_core_1.shell.cp('-Rf', path_1.default.join(paths.apiImplHull, 'android/{.*,*}'), outputDirectory);
            const srcOutputDirectory = path_1.default.join(outputDirectory, 'lib', SRC_MAIN_JAVA_DIR);
            let pluginPath;
            for (pluginPath of pluginsPaths) {
                const pluginConfig = await ern_core_1.manifest.getPluginConfig(pluginPath, 'android');
                if (pluginConfig) {
                    ern_core_1.log.debug(`Copying ${pluginPath.name} to ${outputDirectory}`);
                    this.copyPluginToOutput(paths, srcOutputDirectory, pluginPath, pluginConfig);
                }
            }
            const editableFiles = await this.generateRequestHandlerClasses(apiDependency, paths, apis);
            await this.updateGradleProperties(paths, reactNativeVersion, outputDirectory);
            await this.updateBuildGradle(paths, reactNativeVersion, outputDirectory);
        }
        finally {
            ern_core_1.shell.popd();
        }
    }
    copyPluginToOutput(paths, pluginOutputDirectory, pluginPath, pluginConfig) {
        if (pluginPath.name === 'react-native') {
            return;
        }
        ern_core_1.log.debug(`injecting ${pluginPath.name} code.`);
        const pluginSrcDirectory = path_1.default.join(paths.outDirectory, 'node_modules', pluginPath.name, 'android', pluginConfig.moduleName, SRC_MAIN_JAVA_DIR, '*');
        fs_extra_1.default.ensureDirSync(pluginOutputDirectory);
        ern_core_1.log.debug(`Copying code from ${pluginSrcDirectory} to ${pluginOutputDirectory}`);
        ern_core_1.shell.cp('-Rf', pluginSrcDirectory, pluginOutputDirectory);
    }
    updateBuildGradle(paths, reactNativeVersion, outputDirectory) {
        let mustacheView = {};
        const versions = ern_core_1.android.resolveAndroidVersions({
            androidGradlePlugin: '3.2.1',
            reactNativeVersion,
        });
        mustacheView.reactNativeVersion = reactNativeVersion;
        mustacheView = Object.assign(mustacheView, versions);
        ern_core_1.mustacheUtils.mustacheRenderToOutputFileUsingTemplateFile(path_1.default.join(paths.apiImplHull, 'android/build.gradle'), mustacheView, path_1.default.join(outputDirectory, 'build.gradle'));
        return ern_core_1.mustacheUtils.mustacheRenderToOutputFileUsingTemplateFile(path_1.default.join(paths.apiImplHull, 'android/lib/build.gradle'), mustacheView, path_1.default.join(outputDirectory, 'lib/build.gradle'));
    }
    updateGradleProperties(paths, reactNativeVersion, outputDirectory) {
        let mustacheView = {};
        const versions = ern_core_1.android.resolveAndroidVersions({ reactNativeVersion });
        mustacheView = Object.assign(mustacheView, versions);
        return ern_core_1.mustacheUtils.mustacheRenderToOutputFileUsingTemplateFile(path_1.default.join(paths.apiImplHull, 'android/gradle/wrapper/gradle-wrapper.properties'), mustacheView, path_1.default.join(outputDirectory, 'gradle/wrapper/gradle-wrapper.properties'));
    }
    async generateRequestHandlerClasses(apiDependency, paths, apis) {
        ern_core_1.log.debug('=== updating request handler implementation class ===');
        try {
            const editableFiles = [];
            const resourceDir = path_1.default.join(ern_core_1.Platform.currentPlatformVersionPath, 'ern-api-impl-gen/resources/android');
            const outputDir = path_1.default.join(paths.outDirectory, 'android/lib', SRC_MAIN_JAVA_DIR, API_IMPL_PACKAGE);
            fs_extra_1.default.ensureDirSync(outputDir);
            for (const api of apis) {
                const { files, classNames } = ApiImplAndroidGenerator.getMustacheFileNamesMap(resourceDir, api.apiName);
                for (const file of files) {
                    if (file === 'requestHandlerProvider.mustache') {
                        editableFiles.push(path_1.default.join(outputDir, classNames[file]));
                        if (this.regenerateApiImpl) {
                            ern_core_1.log.debug(`Skipping regeneration of ${classNames[file]}`);
                            continue;
                        }
                    }
                    if (classNames[file]) {
                        const partialProxy = (name) => {
                            return fs_extra_1.default.readFileSync(path_1.default.join(resourceDir, `${name}.mustache`), 'utf8');
                        };
                        await ern_core_1.mustacheUtils.mustacheRenderToOutputFileUsingTemplateFile(path_1.default.join(resourceDir, file), api, path_1.default.join(outputDir, classNames[file]), partialProxy);
                    }
                    else {
                        ern_core_1.log.info(`No mapping for for ${file}. Mustaching skipped.`);
                    }
                }
                ern_core_1.log.debug(`Api implementation files successfully generated for ${api.apiName}Api`);
            }
            return editableFiles;
        }
        catch (e) {
            throw new Error(`Failed to update RequestHandlerClass: ${e}`);
        }
    }
}
exports.default = ApiImplAndroidGenerator;
//# sourceMappingURL=ApiImplAndroidGenerator.js.map