"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ROOT_DIR = void 0;
const ern_core_1 = require("ern-core");
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const xcode_ern_1 = __importDefault(require("xcode-ern"));
const fs_readdir_recursive_1 = __importDefault(require("fs-readdir-recursive"));
exports.ROOT_DIR = ern_core_1.shell.pwd();
const API_IMPL_GROUP_NAME = 'APIImpls';
class ApiImplIosGenerator {
    static getMustacheFileNamesMap(resourceDir, apiName) {
        const files = (0, fs_readdir_recursive_1.default)(resourceDir, (f) => f.endsWith('.mustache'));
        const classNames = {
            'RequestHandlerConfig.swift.mustache': 'RequestHandlerConfig.swift',
            'RequestHandlerProvider.swift.mustache': 'RequestHandlerProvider.swift',
            'apiController.mustache': `${apiName}ApiController.swift`,
            'requestHandlerDelegate.mustache': `${apiName}ApiRequestHandlerDelegate.swift`,
            'requestHandlerProvider.mustache': `${apiName}ApiRequestHandlerProvider.swift`,
        };
        return { files, classNames };
    }
    get name() {
        return 'ApiImplIosGenerator';
    }
    get platform() {
        return 'ios';
    }
    async generate(apiDependency, paths, reactNativeVersion, plugins, apis, regen) {
        ern_core_1.log.debug(`Starting project generation for ${this.platform}`);
        this.regenerateApiImpl = regen;
        await this.fillHull(paths, plugins, apis, reactNativeVersion);
    }
    async fillHull(paths, plugins, apis, reactNativeVersion) {
        try {
            const pathSpec = {
                outputDir: path_1.default.join(paths.outDirectory, 'ios'),
                projectHullDir: path_1.default.join(paths.apiImplHull, 'ios/{.*,*}'),
                rootDir: exports.ROOT_DIR,
            };
            const projectSpec = {
                deploymentTarget: ern_core_1.iosUtil.getDefaultIosDeploymentTarget(reactNativeVersion),
                nodeModulesRelativePath: '../node_modules',
                projectName: 'ElectrodeApiImpl',
            };
            const { iosProject, projectPath } = await ern_core_1.iosUtil.fillProjectHull(pathSpec, projectSpec, plugins);
            await this.generateRequestHandlerClasses(iosProject, pathSpec, projectSpec, apis);
            fs_extra_1.default.writeFileSync(projectPath, iosProject.writeSync());
            ern_core_1.log.debug('[=== Completed api-impl hull filling ===]');
        }
        catch (e) {
            ern_core_1.log.error('Error while generating api impl hull for ios');
            throw e;
        }
    }
    async getIosApiImplProject(apiImplProjectPath) {
        ern_core_1.log.debug(apiImplProjectPath);
        const containerProject = xcode_ern_1.default.project(apiImplProjectPath);
        return new Promise((resolve, reject) => {
            containerProject.parse((err) => {
                if (err) {
                    reject(err);
                }
                resolve(containerProject);
            });
        });
    }
    async generateRequestHandlerClasses(iosProject, pathSpec, projectSpec, apis) {
        ern_core_1.log.debug('=== updating request handler implementation class ===');
        const resourceDir = path_1.default.join(ern_core_1.Platform.currentPlatformVersionPath, 'ern-api-impl-gen/resources/ios');
        const outputDir = path_1.default.join(pathSpec.outputDir, projectSpec.projectName, API_IMPL_GROUP_NAME);
        fs_extra_1.default.ensureDirSync(outputDir);
        iosProject.addSourceFile(path_1.default.join(API_IMPL_GROUP_NAME, 'RequestHandlerConfig.swift'), null, iosProject.findPBXGroupKey({ name: API_IMPL_GROUP_NAME }));
        iosProject.addSourceFile(path_1.default.join(API_IMPL_GROUP_NAME, 'RequestHandlerProvider.swift'), null, iosProject.findPBXGroupKey({ name: API_IMPL_GROUP_NAME }));
        const editableFiles = [];
        for (const api of apis) {
            const { files, classNames } = ApiImplIosGenerator.getMustacheFileNamesMap(resourceDir, api.apiName);
            for (const file of files) {
                if (file === 'requestHandlerProvider.mustache') {
                    const reqHandlerProviderClassFile = path_1.default.join(outputDir, classNames[file]);
                    editableFiles.push(reqHandlerProviderClassFile);
                    if (this.regenerateApiImpl &&
                        ern_core_1.shell.test('-e', reqHandlerProviderClassFile)) {
                        ern_core_1.log.debug(`Skipping regeneration of ${classNames[file]}`);
                        continue;
                    }
                }
                if (classNames[file]) {
                    const partialProxy = (name) => {
                        return fs_extra_1.default.readFileSync(path_1.default.join(resourceDir, `${name}.mustache`), 'utf8');
                    };
                    await ern_core_1.mustacheUtils.mustacheRenderToOutputFileUsingTemplateFile(path_1.default.join(resourceDir, file), api, path_1.default.join(outputDir, classNames[file]), partialProxy);
                    iosProject.addSourceFile(path_1.default.join(API_IMPL_GROUP_NAME, classNames[file]), null, iosProject.findPBXGroupKey({ name: API_IMPL_GROUP_NAME }));
                }
                else {
                    ern_core_1.log.info(`No mapping for for ${file}. Mustaching skipped.`);
                }
            }
            ern_core_1.log.debug(`Api implementation files successfully generated for ${api.apiName}Api`);
        }
    }
}
exports.default = ApiImplIosGenerator;
//# sourceMappingURL=ApiImplIosGenerator.js.map